﻿/*
 * XBee.h
 *
 * 宣言のみ集めてみました。
 * Created: 2012/12/08 0:20:29
 *  Author: morishita
 *
 *  History: 2012/12/16 802.15.4に対応（デバッグはしていないので、動くかは不明）
 *                      もし802.15.4を利用する場合は、全てのXBee関連ファイルをインクルードする前にXBEE_MODELE_802_15_4を#defineで宣言して下さい。
 *           2013/03/11  ／／によるコメントアウトがコメントとして認識されてない行があったので/＊（本当は半角）＊/に変更した。
 *                       実機でテストしたところAPIパケットでの受信時と送信時でペイロードサイズが異なっていたのを反映させた。
 *                      これはDegiMesh F/Wのバグだと思う。
 *                      リテラルデータをdefineによる定数に置き換えた。
 */ 


#ifndef XBEE_H_
#define XBEE_H_

//------header file include------------------------------------------------------
#include "fifo.h"
#include <stdbool.h>

//------constant value-----------------------------------------------------------
#define	START_DELIMITER_CHAR                0x7E        /* API通信モード時のヘッダ */
#define	START_ESCAPE_CHAR                   0x7D        /* API2モード通信時に使用するエスケープ文字 */
#define	BROADCAST_ADDRESS_MSB               0x00000000ul/* ブロードキャストモード時のアドレス */
#define	BROADCAST_ADDRESS_LSB               0x0000FFFFul
#define	PACKET_ACKNOWLEDGED                 0x01
#define	BROADCAST_PACKET                    0x02
#define UNICAST_TRANSMIT                    0
#define BROADCAST_TRANSMIT                  1
// 送信ステータスに使用される（ZBの場合はもっと多いが、省略している）
#define	SUCCESS                             0x00        /* DigiMesh, ZB, 802.15.4 で定義 */
#define	CCA_FAILURE                         0x02        /* DigiMesh, ZB, 802.15.4 で定義 */
#define	PURGED                              0x03        /* 802.15.4 で定義 */
#define	INVALID_DESTINATION_ENDPOINT        0x15        /* DigiMesh, ZB で定義 */
#define	NETWORK_ACK_FAILURE                 0x21        /* DigiMesh, ZB で定義 */
#define	NOT_JOINED_TO_NETWORK               0x22        /* ZB で定義 */
#define	SELF_ADDRESSED                      0x23        /* ZB で定義 */
#define	ADDRESS_NOT_FOUND                   0x24        /* ZB で定義 */
#define	ROUTE_NOT_FOUND                     0x25        /* DigiMesh, ZB で共通 */
// API Identifier values（全ては宣言していない）
#define	AT_COMMAND							0x08
#define	AT_COMMAND_RESPONSE					0x88
#define	AT_COMMAND_QUEUE_PARAMETER_BALUE	0x09
#define	REMOTE_AT_COMMAND_RQ				0x17
#ifdef XBEE_MODELE_802_15_4
    #define	TRANSMIT_STATUS                 0x89
    #define	TRANSMIT_RQ                     0x00
    #define	RX_INDICATOR                    0x80
#else
    #define	TRANSMIT_STATUS                 0x8B
    #define	TRANSMIT_RQ                     0x10
    #define	RX_INDICATOR                    0x90        /* AO == 0のときの識別子 */
    #define	EXPLICIT_RX_INDICATOR           0x91
#endif


// other
#define API_OFF								0           /* 通信モード */
#define API_WITHOUT_ESCAPES					1
#define API_WITH_ESCAPES					2
#define XBee_API_BUFF_SIZE					180         /* APIデータフレーム解析用のバッファサイズ。 AP==2の時、パケットサイズが最大約2倍になり得ることに注意。 */
#define XBee_BUFF_SIZE						300         /* 受信データを格納するバッファのサイズ。割り込み処理使用時は十分に大きなバッファを用意すること（特に透過モード時）。 */
#define XBee_API_PACKET_MAX_SIZE_FOR_TX     72          /* 送信ペイロードサイズ。ZBではパケットサイズは設定によって変わるので、ATNPコマンドによって読み出す必要がある。また、802.15.4では100までOKだが、面倒なので72に固定とする。 */
#define XBee_API_PACKET_MAX_SIZE_FOR_RX     73          /* 受信ペイロードサイズ。ZBではパケットサイズは設定によって変わるので、ATNPコマンドによって読み出す必要がある。本来は72のはずだが・・XBeeのバグか？ */
#define XBee_API_RX_LENGTH_MAX              85          /* API受信フレームに含まれるlengthの最大値 for Digimesh（@2013/3/11最新版）, 72+12で通常は84までのはずだが、XBeeのF/Wのバグのせいか、1つ多くてもOKの様だ。ここで、84は長さを表しており、AO == 0を想定した場合の数字。 AO == 1なら、90 */
#define XBee_AT_COMMAND_PARAM_SIZE          10

//------構造体-----------------------------------
/*************************************************************************/
/* XBeeのアドレス                                                        */
/*************************************************************************/
typedef struct{
    unsigned long int       msb;                        // アドレス上位バイト。
    unsigned long int       lsb;                        // アドレス下位バイト。
}XBEE_ADD;
/*************************************************************************/
/* 他ノードからの通信パケット                                            */
/* Recieve Data Packet                                                   */
/*************************************************************************/
typedef struct{
    XBEE_ADD                add;                                        // 送信元アドレス
    unsigned int            networkAddress;                             // ネットワークアドレス
    char                    option;                                     // option, 0x01:packet acknowledged , 0x02:packet is a broadcast packet.
    int                     size;
    char                    dataPacket[XBee_API_PACKET_MAX_SIZE_FOR_RX + 1];   // 純粋に文字データの場合もあるので、最後をヌルで占めるために一つ余計に確保する。
}XBEE_RDP;
/************************************************************************/
/* 送信時の、送信側モジュールからのレスポンスパケット                   */
/* Trasmit Status                                                       */
/************************************************************************/
typedef struct{
    char 					frameId;
    unsigned int			networkAddress;
    char 					TransmissionRetryCount;
    char 					deliveryStatus;
    char 					discoveryStatus;
}XBEE_TS;
/************************************************************************/
/* ATコマンドのレスポンス                                               */
/* AT Cmmand Response                                                   */
/************************************************************************/
typedef struct{
    int                     length;
    char                    frameId;
    char                    commandNameH;               // command name: two ascii characters that identify the AT command
    char                    commandNameL;
    char                    status;
    char                    param[XBee_AT_COMMAND_PARAM_SIZE];
    int                     paramSize;
}XBEE_ACR;
/************************************************************************/
/* ATコマンドのレスポンス                                               */
/* AT Cmmand Response Recieve Result and Setting                        */
/************************************************************************/
typedef struct{
    bool                    timeOut;                    // true: タイムアウトしたことを示す
    XBEE_ACR                acr;
}XBEE_ACRRRS;
/*************************************************************************/
/* パケット情報と処理中のフラグ類を集めた構造体                          */
/*************************************************************************/
typedef struct{
    char                    apiId;                      // API Identifer（APIフレームの識別子）
    int                     length;                     // パケットの長さ情報（packet[]へ格納する長さではないことに注意）
    bool                    ap2MaskFlag;                // AP == 2の場合に、エスケープ文字を受信していることを示すフラグ.
    int                     wp;
    char                    buff[XBee_API_BUFF_SIZE];   // パケット解析用バッファ
}XBEE_RP;
/*************************************************************************/
/* 送信設定メンバ                                                        */
/*************************************************************************/
typedef struct{
    XBEE_ADD                add;                        // 送信先アドレス
    unsigned int            networkAddress;             // ネットワークアドレス
    //int                   cluster_id;                 // AO == 1で使用するが、未使用の予定
    //int                   profile_id;                 // AO == 1で使用するが、未使用の予定
    char                    broadcastRadius;            // ブロードキャスト半径（ブロードキャストメッセージのリレー数）
    char                    apiEnable;                  // XBeeのAPレジスタのこと。通信モードがAPIモードなのかATモードなのかを表す
    char                    transmitType;               // 送信をユニキャストかブロードキャストにするか選択する
}XBEE_TX_SETTING;
/*************************************************************************/
/* 送信用ワーキングスペース用                                            */
/* 外部には無用                                                          */
/*************************************************************************/
typedef struct{
    int						checkSum;					// チェックサムです。
    char                    frameId;                    // Frame ID（個々のコマンドの識別用）
    FIFO                    buf1;                       // printfやputs用にバッファを用意 buf2よりbuf1の方が大きい方が良い
    FIFO                    buf2;                       // AVRマイコンで省RAM設定なら1,2のどちらも使うが、その他のマイコンでは2は不使用
}XBEE_WS_TX;
/*********************************************************************************************/
/* 処理にロックがかかっている状態で受信データを取りこぼさないようにするための退避用バッファ  */
/* 外部には無用                                                                              */
/* Temporary Work Space                                                                      */
/*********************************************************************************************/
typedef struct{
    bool                    overflow;                   // true: オーバーフローしました。
    int                     wp;
    char                    buff[10];
}XBEE_TWS;
/*************************************************************************/
/* 受信用ワーキングスペース用                                            */
/* 外部には無用                                                          */
/*************************************************************************/
typedef struct{
    bool                    lock;                       // true: パラメータを書き込み不可。
    bool                    busy;                       // true: 割り込み処理中
    XBEE_TWS                tws;                        // 割り込みが発生する前に処理が追いつかない場合はこのバッファへデータを一時的に退避する
    int                     packetCounter;              // パケットを受信する毎に(packetCounter + 1) % 30000とする。
    XBEE_ADD                backupAdd;                  // 送信元アドレスのバックアップ
    int                     nodeCounter;                // 異なるノードからの信号を受信するたびに(nodeCounter + 1) % 30000とする。
    XBEE_RP                 packet;
    FIFO                    payload;
    char                    apiEnable;                  // XBeeのAPレジスタのこと。通信モードがAPIモードなのかATモードなのかを表す
    XBEE_TS                 ts;                         // Transmit Status
    XBEE_ACRRRS             atcr;                       // AT command response+
    XBEE_RDP                rdp;                        // 受信データパケット
    XBEE_ADD                addMask;                    // ここにセットしたアドレス以外の受信データをバッファへコピーしなくなります。無効にする場合は、ブロードキャストアドレスを入力してください。
}XBEE_WS_RX;
/*************************************************************************/
/* 受信パケットの一時的な解析結果を格納する構造体                        */
/* パケット位置が不明なバッファを解析にかける場合の利用を想定しています。*/
/* Temporary API Packet                                                  */
/*************************************************************************/
typedef struct{
    char                    apiId;                      // API Identifer（APIフレームの識別子）
    int                     packetSize;                 // パケットとして格納し直されたデータ長
    int                     start;                      // 被解析バッファのパケット開始位置
    int                     end;                        // 被解析バッファのパケットの終了位置
    char                    buff[XBee_API_BUFF_SIZE];   // 格納しなおされたパケット（エスケープ文字等の処理済み）
}XBEE_TAP;
/*************************************************************************/
/* ノードの座標                                                          */
/*************************************************************************/
typedef struct{
    long int                lat;                        // latitude
    long int                lon;                        // longitude
}XBEE_POS;
/*************************************************************************/
/* ノード情報                                                            */
/*************************************************************************/
typedef struct{
    XBEE_ADD                add;                        // アドレス
    XBEE_POS                pos;                        // 座標（単位は任意. 森下は0.0001分単位で使います.）
    char                    Rssi;                       // 受信信号強度
    bool                    connection;                 // true: 接続中（少なくとも電波は届く範囲にいる。）
}XBEE_NODE;
/*************************************************************************/
/* 他ノードから受信した通信パケットとノード情報のセット                  */
/*************************************************************************/
typedef struct{
    XBEE_NODE               node;                                               // ノード情報
    int                     size;                                               // パケットサイズ
    char                    dataPacket[XBee_API_PACKET_MAX_SIZE_FOR_RX + 1];    // 純粋に文字データの場合もあるので、最後をヌルで占めるために一つ余計に確保する。
}XBEE_RF_PACKET;


//------汎用関数を外部から利用できるようにextern--------------------------------------
// XbeeControl.c
extern void XBee_SetTransmitMode(char transmit_mode);
extern void XBee_SetTxSetting(unsigned long int address_msb, unsigned long int address_lsb, unsigned int network_add, char radius, char transmit_mode);
extern void XBee_init(unsigned long int, unsigned long int, unsigned int, char, char, char, char *, int, char *, int);
extern void XBee_init2(unsigned long int address_msb, unsigned long int address_lsb, unsigned int network_add, char radius, char transmit_mode, char AP);
extern void XBee_send1(char *buff, int size, unsigned long int add_msb, unsigned long int add_lsb);
extern void XBee_send2(char *, int);
extern void XBee_send3(char *);
extern char XBee_getc(void);                                // 受信済みのデータを1 byte返します。受信データがない場合は0x00を返します。
extern int  XBee_gets(char *buff);                          // 受信済みのデータを受け取ります。バッファオーバーランの可能性がありますがシステムは関知しません。（最悪、暴走します）
extern int  XBee_getsb(char *buff, int size);               // 受け取りデータ数を指定して、受信済みのデータを受け取ります。格納先のバッファのサイズを指定していればバッファオーバーランの心配がありません。
extern int  XBee_buff_check(void);                          // バッファをチェックし、受信データサイズを返す（副作用として、非割り込み処理時に受信済みのシリアルデータを処理します。）
extern char XBee_GetRssi(void);                             // 直前に他のノードから受信した信号の受信信号強度を返す。
extern XBEE_ACRRRS XBee_GetAtcResult(void);                 // ATコマンドレスポンスの処理結果を返す。
extern XBEE_TS XBee_GetTransmitStatus(void);                // 直前の送信の結果を返す（ただし、本関数実行時には未だ送信結果を受信していない可能性もあります。この対策は関数の実装部のコメントを参照のこと。）
extern char XBee_WaitApiPacket(char apiId);                 // 任意のAPIパケットを受信するまで待つ。
extern int  XBee_GetPacketCounter(void);                    // 受信したパケット数を返す。ある閾値以上の受信が行われると0クリアされる。
extern int  XBee_GetNodeCounter(void);                      // 他のノードからRFデータを受信した回数（続けて同じノードから受信した場合はカウントしない）
extern XBEE_NODE XBee_GetNodeAddAndRssi(void);              // 直前に受信したRFデータの送信元ノードのアドレスとRSSIを返す。
extern void XBee_SetAddMask(XBEE_ADD *add);                 // 受信するノードのアドレスを限定する（通信相手を限定するのに使用します）。
// XbeeNodeManager.c
extern void Xbee_InitManager(void);                         // マネージャ情報を初期化する
extern XBEE_RF_PACKET Xbee_GetOnePacket(void);              // ユーザー側へ、1パケット分のデータを返す
extern void Xbee_SetAddressAndPacketSize(XBEE_ADD *add, int size);    // XbeeControl.cからの呼び出しを想定しています。
extern void Xbee_GivePullOutNum(int size);                  // XbeeControl.cからの呼び出しを想定しています。
extern int  XBee_GetRfPacketNum(void);                      // 未読の受信RFパケット数を返す
// XbeeApiTxDataCreator.c
extern void Xbee_PutcDummy(char c);                         // システム用（ユーザは呼び出さないでください。）

#endif /* XBEE_H_ */