﻿/*
 * XBeeApiParser.c
 * 受信したパケットを解析する関数群
 * シリアルの割り込みやシリアル送受信系の関数とは無縁なので、分離独立させた。
 * Created: 2012/12/08 16:29:06
 *  Author: morishita
 * 
 * 2013/03/11   XBee_ParseApiReceivePacket()にて、エラーでreturnする場合にバッファ内のデータを初期化していなかったのを訂正した。
 */ 
#ifndef XBEE_API_PARCER_C_
#define XBEE_API_PARCER_C_

//------header file include-----------------------------------------------
#include "XBee.h"
#include <stdbool.h>

//------汎用関数を外部から利用できるようにextern--------------------------
extern bool XBee_IsEscape(char c);
extern XBEE_TAP XBee_ParseApiPacket(char *buff, int size, char ApiMode);
extern void XBee_ParseApiATCommandResponse(char *buff, XBEE_ACR *acr);
extern void XBee_ParseTransmitStatus(char *buff, XBEE_TS *ts);
extern void XBee_ParseApiReceivePacket(char *buff, XBEE_RDP *rdp);
extern char XBee_ParseApiPacketc(char c, XBEE_RP *recievePacket, char ApiMode);  // XbeeControl.cからの呼び出しに備えています。ユーザーが呼び出すことは禁止します。

//------以下、関数宣言----------------------------------------------------
/************************************************************************/
/* 引数で受け取った１文字がエスケープ文字かどうかを検査する             */
/*                                                                      */
/* AP == 2の設定時にエスケープ文字として扱われるかをチェックします。    */
/*                                                                      */
/* 引数: c      : 検査したいデータ                                      */
/* return value : true: c is escape char.                               */
/************************************************************************/
bool XBee_IsEscape(char c)
{
    if(c == 0x7e || c == 0x7d || c == 0x11 || c == 0x13)
        return true;
    else
        return false;
}

/************************************************************************/
/* APIパケットで送られてきた、ATコマンドの応答を解析する                */
/*                                                                      */
/* XBee_ParseApiPacketc()にて処理済みのバッファであること及び、         */
/* 目的にあったパケットであることを前提に処理を行います。               */
/* ユーザが呼び出す際は、XBee_ParseApiPacket()より受け取った            */
/* XBEE_TAP構造体オブジェクトのバッファを渡すようにして下さい。         */
/* なお、処理を高速化するため結果はポインタを使った操作で提供され、     */
/* 結果は引数に格納されます。                                           */
/* argument:     *buff: パケットを格納したバッファの先頭アドレス        */
/*               *acr : 解析結果                                        */
/************************************************************************/
void XBee_ParseApiATCommandResponse(char *buff, XBEE_ACR *acr)
{
    int i = 0;

    acr->paramSize      = 0;
    acr->length         = (int)(*(buff + 2));
    acr->frameId        = *(buff + 4);
    acr->commandNameH   = *(buff + 5);
    acr->commandNameL   = *(buff + 6);
    acr->status         = *(buff + 7);
    for (i = 0; i < XBee_AT_COMMAND_PARAM_SIZE; i++) *(acr->param + i) = 0x00;
    if(acr->commandNameH == 'D' && acr->commandNameL == 'B')    // Received Signal Strength
    {
        *(acr->param) = *(buff + 8);
        acr->paramSize = 1;
    }else{
        // na
    }
	
    return;
}

/************************************************************************/
/* APIパケットで送られてきた、APIパケット送信結果を解析する             */
/*                                                                      */
/* XBee_ParseApiPacketc()にて処理済みのバッファであること及び、         */
/* 目的にあったパケットであることを前提に処理を行います。               */
/* ユーザが呼び出す際は、XBee_ParseApiPacket()より受け取った            */
/* XBEE_TAP構造体オブジェクトのバッファを渡すようにして下さい。         */
/* なお、処理を高速化するため結果はポインタを使った操作で提供され、     */
/* 結果は引数に格納されます。                                           */
/* argument:     *buff: パケットを格納したバッファの先頭アドレス        */
/*               *ts   : 解析結果                                       */
/************************************************************************/
void XBee_ParseTransmitStatus(char *buff, XBEE_TS *ts)
{
	ts->frameId = *(buff + 4);
	ts->networkAddress = ((unsigned int)(*(buff + 7)) << 8) + (unsigned int)(*(buff + 7));
	ts->TransmissionRetryCount = *(buff + 7);
	ts->deliveryStatus = *(buff + 8);
	ts->discoveryStatus = *(buff + 9);
	return;
}

/*************************************************************************/
/* APIパケットで送られてきた、他のノードからのメッセージを解析する       */
/*                                                                       */
/* XBee_ParseApiPacketc()にて処理済みのバッファであること及び、          */
/* 目的にあったパケットであることを前提に処理を行います。                */
/* ユーザが呼び出す際は、XBee_ParseApiPacket()より受け取った             */
/* XBEE_TAP構造体オブジェクトのバッファを渡すようにして下さい。          */
/* なお、処理を高速化するため結果はポインタを使った操作で提供され、      */
/* 結果は引数に格納されます。                                            */
/* argument:     *buff: パケットを格納したバッファの先頭アドレス         */
/*               *rdp : 解析結果                                         */
/*************************************************************************/
void XBee_ParseApiReceivePacket(char *buff, XBEE_RDP *rdp)
{
    int 	i, wp = 0, length;
    //char	option;

    length = (int)*(buff + 2);
    rdp->size = length - 12;
    rdp->add.msb = 0ul;
    rdp->add.lsb = 0ul;
    if(length > XBee_API_RX_LENGTH_MAX)  // 不正パケット。AO == 0を想定, AO == 1なら、90（のはず）
    {
        rdp->size = 0;
        rdp->dataPacket[0] = 0x00;
        rdp->networkAddress = 0u;
        rdp->option = 0x00;
        return;
    }    
    i = 4;
    while(i)
    {
        rdp->add.msb += ((unsigned long int)(*(buff + 8 - i)) << (8 * (i - 1)));	// 送信元アドレスの格納
        i--;
    }
    i = 4;
    while(i)
    {
        rdp->add.lsb += ((unsigned long int)(*(buff + 12 - i)) << (8 * (i - 1)));
        i--;
    }
    // 12,13はネットワークアドレス・・・ではなく、予備（データシート90000991_Eによれば）
    rdp->networkAddress = ((unsigned int)(*(buff + 12)) << 8) + (unsigned int)(*(buff + 13));
    // Option
    rdp->option = *(buff + 14);                 // option,0x01:packet acknowledged , 0x02:packet is a broadcast packet.
    // RF data
    i = 15;                                     // バッファの内容をコピーする
    while(i <= length + 2)                      // 最後はチェックサムなので、入れない。
        rdp->dataPacket[wp++] = buff[i++];
    rdp->dataPacket[wp] = 0x00;                 // 受信データが文字列の場合にヌルで閉めると便利
    return;
}

/***************************************************************************************************************************/
/* 受信したパケットデータを一文字ずつ渡すことで処理を実施し、パケットを発見次第パケットの種類を通知します。                */
/*                                                                                                                         */
/* XBee_ParseApiPacket()とXBee_RxInterrupt()からの呼び出しを想定しています。                                               */
/* バッファ内容を一文字ずつ渡してください。                                                                                */
/* 処理途中の経過は引数のwsrx構造体内に保存します。                                                                        */
/* エスケープ文字の処理も実施します。                                                                                      */
/*                                                                                                                         */
/* 注意：XbeeControl.cからの呼び出しに備えて外部へ公開しますが、ユーザーが呼び出すことは禁止します。                       */
/*                                                                                                                         */
/* argument:  c             : 解析したい文字                                                                               */
/*            *recievePacket: 処理中の途中経過を格納する構造体                                                             */
/*            ApiMode       : APIモード（AP == 0（透過モード）, 1, 2）                                                     */
/* return value: パケットの候補を処理中は0xff，パケット候補の未発見時は0x00，パケットを発見した際はパケットIDを返す。      */
/***************************************************************************************************************************/
char XBee_ParseApiPacketc(char c, XBEE_RP *recievePacket, char ApiMode)
{
    int		i, check_sum;
    char 	ans = 0;
    bool    found = false;
    
    if(recievePacket->wp >= XBee_API_BUFF_SIZE) recievePacket->wp = 0;  // バッファが一杯なら、解析できないので0クリア
    if(ApiMode == API_WITH_ESCAPES)                                     // AP == 2への対応
    {
        if(c == START_DELIMITER_CHAR)
            recievePacket->wp = 0;
        else if(c == START_ESCAPE_CHAR)                                 // 0x7dは次の文字が0x20でマスクされていることを示している
            recievePacket->ap2MaskFlag = true;
        else if(recievePacket->ap2MaskFlag == true)
        {
            c = 0x20 ^ c;
            recievePacket->ap2MaskFlag = false;
        }
    }
    if(recievePacket->ap2MaskFlag == false)
    {
        if(recievePacket->wp == 0)
        {
            if(c != START_DELIMITER_CHAR)                               // フレームヘッダのチェック
                recievePacket->wp = 0;
            else
                recievePacket->buff[recievePacket->wp++] = c;
        }
        else if(recievePacket->wp == 1)                                 // lenghtの上位バイト パケット構造から必ず0となる
        {
            if(c != 0x00)
                recievePacket->wp = 0;
            else
                recievePacket->buff[recievePacket->wp++] = c;
        }
        else if(recievePacket->wp == 2)                                 // lenghtの下位バイト
        {
            if(c > XBee_API_RX_LENGTH_MAX)
                recievePacket->wp = 0;
            else{
                recievePacket->buff[recievePacket->wp++] = c;
                recievePacket->length = (int)c;                         // lengthを取得
            }
        }
        else if(recievePacket->wp == 3)                                // コマンドの識別子を取得
        {
            if(c == REMOTE_AT_COMMAND_RQ || c == AT_COMMAND_RESPONSE || c == TRANSMIT_STATUS || c == RX_INDICATOR || c == EXPLICIT_RX_INDICATOR)// API IDのチェック 2010/11/15現在、全てのIDの確認はしていないはずだ
            {
                recievePacket->buff[recievePacket->wp++] = c;
                recievePacket->apiId = c;
            }
            else
                recievePacket->wp = 0;
        }
        else
        {                                                               // その他のデータをとにかくバッファに入れる。送信ノードのアドレス等も含まれる。
            recievePacket->buff[recievePacket->wp++] = c;
            if(recievePacket->wp > (recievePacket->length + 3))         // 左の条件を満たせば、チェックサムまで読み込んでいるはず
            {
                check_sum = 0;
                for(i = 3; i < (recievePacket->wp - 1); i++)            // チェックサムの計算
                {
                    check_sum += (int)recievePacket->buff[i];
                    check_sum &= 0x00ff;
                }
                check_sum = 0xff - check_sum;
                if(c == (char)check_sum)                                // チェックサムの一致を確認
                {
                    found = true;
                    ans = recievePacket->apiId;
                }
                recievePacket->wp = 0;                                  // 次回の処理に備えて0クリア
            }
        }
    }
    if(found == false && recievePacket->wp > 0) ans = 0xff;             // パケットが見つかっていない段階で、かつ処理が進行中ならば0xffを返す
    return ans;
}

/************************************************************************/
/* 渡されたデータ系列を走査し、解析結果を返す                           */
/*                                                                      */
/* 最初に発見したパケット構造の結果を報告します。                       */
/*                                                                      */
/* argument:    buff   : 解析対象バッファの先頭アドレス                 */
/*              size   : 解析対象バッファのサイズ                       */
/*              ApiMode: APIモード（AP == 0（透過モード）, 1, 2）       */
/* return value: 処理結果                                               */
/************************************************************************/
XBEE_TAP XBee_ParseApiPacket(char *buff, int size, char ApiMode)
{
    XBEE_RP recievePacket;
    XBEE_TAP ans;
    int i, start = 0, end = 0;
    char apiId = 0x00;
    bool flag = false;
    
    recievePacket.wp = 0;
    for (i = 0; i < size; i++)
    {
        apiId = XBee_ParseApiPacketc(*(buff + i), &recievePacket, ApiMode);
        if(apiId == 0x00)
        {
            start = 0;
            end = 0;
            flag = false;
        }
        else
        {
            if(flag != true) 
            {
                flag = true;
                start = i;
            }
            if(apiId != 0xff) end = i;
        }
        if (end != 0) break;
    }
    // 返すのは、処理済みの受信データ、解析開始位置と終了位置
    ans.apiId = apiId;
    ans.start = start;
    ans.end = end;
    ans.packetSize = recievePacket.wp;
    for(i = 0; i < recievePacket.wp && i < (int)sizeof(ans.buff); i++) 
        ans.buff[i] = recievePacket.buff[i];
    return ans;
}


#endif
