﻿/*
 * XbeeApiTxDataCreator.c
 * 送信用のパケット作成を行うモジュールです。
 * 本ソースコードをインクルードすると、XbeeControl.cによる送信動作が行われなくなり、代わりにAPI送信パケットの作成が可能になります。
 * 送信先の設定はXbeeControl.cの初期化・設定関数を呼び出して実施してください。
 * 
 * Created: 2012/12/16 6:50:55
 *  Author: morishita
 */ 

#ifndef XBEE_TX_DATA_CREATOR_H_
#define XBEE_TX_DATA_CREATOR_H_

//------header file include------------------------------------------------------
#include "XBee.h"

//------global variable number---------------------------------------------------
static char *_api;                  // 作成するAPIパケットデータを格納するポインタ。ユーザがセットする前に格納が始まるとまずい。
static int  _api_size;
static int  _wp;
static bool _lock = true;

/************************************************************************/
/* ユーザにセットされたバッファへデータを格納していきます。             */
/* XbeeControl.cの_sci_putc()からの呼び出しを想定しています。           */
/* ユーザは本関数を呼び出さないでください。                             */
/************************************************************************/
void Xbee_PutcDummy(char c)
{
    if (_lock == false && _wp < _api_size)
        _api[_wp++] = c;
    
    return;
}

/************************************************************************************/
/* 送信用のパケット文字列を返す                                                     */
/*   送信先の設定はXbeeControl.cの初期化・設定関数を呼び出して実施してください。    */
/*                                                                                  */
/*   argument：                                                                     */
/*          api      : APIパケットへ加工された文字列を格納するバッファのポインタ    */
/*          api_size : 上記バッファのサイズ                                         */
/*          tx       : 送信したいデータが格納されたバッファのポインタ               */
/*          tx_size  : 送信したいデータのサイズ                                     */
/*   return value：    パケットサイズ                                               */
/************************************************************************************/
int Xbee_GetTxPacket(char *api, int api_size, char *tx, int tx_size)
{
    _api = api;
    _api_size = api_size;
    _wp = 0;
    _lock = false;
    (void)XBee_send2(tx, tx_size);      // これを実行すると、Xbee_PutcDummy()が呼び出されてデータが格納される。キモイかもしれないが、しょうがない。
    _lock = true;
    return _wp;
}

//------header file include------------------------------------------------------
#include "XbeeControl6.3.c"


#endif