﻿/******************************************************************************************************************************
XBeeのC言語による通信処理プログラム

	auther: 森下功啓 (Katsuhiro Morishita)
	Create: 2009/12/24

	[1]control target(制御対象)：
		XBee DigiMesh(F/W) ver.? @ API transmit mode
		DigiMeshはシリーズ1(S1)にしか実装できませんので注意してください。
    
    [2]developing environment(開発環境)：
        AtmelStudio 6.1 @ 2013/5

    [3]attention(使用上の注意)：
        (1) このファイルをインクルードする前にシリアル関係のファイルもインクルードして下さい。
        (2) シリアル（UART）はXBeeに接続していると仮定している。
        (3) 本プログラムの受信系の処理は、XBeeのハードウェア設定が(AO==0 && (AP == 0 || AP == 1 || AP == 2))の条件の下で稼働します。
        (4) AVRマイコンを利用していて、"moji"の様な文字列によってRAMを圧迫される場合は、
                #define 	TARGET_MICON    AVRmicon_HDR
            と、本ファイルをインクルードする前に定義してみて下さい。
            プログラムメモリからデータを読み出すように動作が変わり、RAMの消費量が著しく減るはずです。
        (5) AVRの使用を宣言した上でのrs_printf()による送信はバッファサイズに制限が有りますので注意して下さい。
        (6) 割り込みを使用する場合、シリアル制御ヘッダファイルを挟むように、XbeeInterruptFor*.*.hをインクルードしてください。
	
    [4]feature and explanation（特徴と説明）：
        (1) 開発コンセプト
        XBee用の専用関数を呼び出したりせずに、“APIパケットを使って”XBeeで無線通信を実現することをコンセプトとしました。
        最も基本的な使用状況では、本ファイル以外のソースコードに手を加えることなく、シリアル有線通信路を無線に置換できます。
        加えて、API通信モードを利用することで透過モードだけでは得られない詳細な通信状況を把握します。

        (2) XBee対応原理
        本ファイルの挿入点以降に記述されているシリアル関係の関数を#define（マクロ）を使ってXBee通信関数へ置換させることで実現しています。
        もちろん、本ファイル内に宣言されている関数を呼び出しても通信は可能です。
        
        (3) 運用方法
        本ソースコードを使用してAPIパケットを送信する場合、マクロをお使いの環境に合わせて設定して下さい。
        “#define SCI_PURGE()        rs_rx_purge()”以下の4行部分および、
        “#define rs_putc(c)         XBee_sendc(c)”以下の部分です。
        送信は通信設定後に利用が可能になります。
        まずはXBee_init()関数で初期化を行ってください。
        その後はrx_puts()関数等を実行すると無線伝送されます。
        
        受信データの処理には、a)割り込みを利用する方法と b)割り込みを使わずに処理する方法そして c)任意の文字列をAPI解析関数を呼び出すことで処理する方法の3つの利用方法を用意しています。
        a) 割り込みを利用する場合、シリアル関係の割り込み処理を記述したヘッダファイルを本ファイルとXbeeInterruptFor*.*.hで挟むようにインクルードしてください。
           その上でXbeeInterruptFor*.*.hの中をよく読んで、シリアル受信処理を記述するようにして下さい。
        b) 割り込みを利用しないけれども受信データの処理をシリアルバッファから自動的に読み込ませたい場合は、適宜XBee_buff_check()（又はrs_rx_purge()）やXBee_getsb()（又はrs_getsb()）等を呼び出してください。
        c) 解析したい文字列をXBee_ParseApiPacket()関数へ渡し下さい。
        
        c)の場合、XBee_init()関数の呼び出しは不要です。


        (4) 以下に示すシリアル通信関数が必要
        ただし最低限必要な関数は、'＊'の付いているものだけです。
        rs_putc()等の関数名は、このプログラムを使用する各人で調整してください。
		
        rs_putc()＊		: 1文字送信関数（APIモードではパケットが冗長になるので頻繁な使用はお勧めできない）	
        rs_puts()		: 文字列出力
        rs_putsb()		: バイナリ出力（引数：char *buff, int data_size）
        rs_getc()＊		: 1文字取得（受信データが無い場合、null(==0)を返すとよいと思います）
        rs_gets()		: 文字列受信
        rs_printf()		: 書式付の文字列出力は、一旦RAM上へ完全に展開して使用するので十分に大きなバッファを指定して欲しい
        rs_rx_buff()＊	: バッファをチェックして受信データがあるかどうかを確認する。受信データがあれば、0以外（普通は受信文字数）を返す

        (5) CTSへの対応
        XBeeのCTS制御へ対応する場合、本ソースコードをインクルードする前に、「XBee_CTS_IO」を定義して下さい。
        例えば、以下のようにして下さい。
        #define     XBee_CTS_IO	PD.PIN.BIT.B7       // XBeeのCTS端子を指定
        #include    "XbeeControl6.3.h"		        // XBeeを制御する関数・他
        当然のことながら、ここで定義したポートはXBeeのCTS端子に電気的に導通しているものとします。
	
	[5]next works：
		(1) そのうち、C++へ移植…するかな？
            オーバーロードが使いたいのでC++へ移行したいとは思っています。
		(2) 継承や、デバッグのためのオブジェクト渡しなど、C言語では実現が難しいこともC++なら簡単だ！
	
	[6]update：
		2009/12/24	この頃に開発開始　構想は以前よりあった。
		2010/2/15	ユニキャストでの任意データの出力に成功。
		2010/2/26	細かい関数構造を変更。受信処理を整備。
		2010/3/2	バッファをヌルで占めるように変更。これでASCII文字データはそのまま出力できるようになった。
		2010/3/31	シリアル受信処理をこちらへ移植。
		2010/11/6	ついに、整備再開。ずいぶん忘れている。受信のための良い手はまだ思いついていない。
		2010/11/11	送信系でひと段落ついた。H8だろうが、AVRだろうが動くはず。
		2010/11/15	mo_sprintf（C言語で云うspritnfに相当）を、文字数制限に対応させることで、一般マイコンでの使用する上でrs_printf()がバッファオーバーランの危険性が無くなる。
					ただし、AVRの使用を宣言した上でのrs_printf()による送信では　まだ　バッファサイズに制限が有りますので注意して下さい。
					AVRマイコンのPSTRに対する対応はしないつもりですのであしからず。
		2010/11/16	受信系が完成。AP == 2と設定していたことで面倒なことになっていた。
					受信プログラムが安全に働くためには、AP == 2の方が良いんだけど、プログラムが面倒なのでAP == 1のときに正常に動くように変更した。
					今後、特に困ったことにならなければ変更はしない予定。
		2011/4/14	ATコマンドモードでの送信プログラムバグを修正。
					また、ATコマンドモード時に'\n'が出力文字中にあった場合は'\r'を追加して出力することで改行させる様に仕様変更。
		2011/4/15	AP == 2への対応化を進める
					SCI_putc()を追加してメンテナンスをやりやすくした。
					rs_rx_purge()の定義も追加
		2011/4/18	+1～+9という文字が送れないし、バックスペースも送信できない。左記のバグを訂正した。
		2011/4/19	Transmit Statusを処理するように変更。
					ATコマンド解析も作りたいなぁ。
		2011/4/20	CTS対応
		2011/4/21	割り込み対応
					バッファを完全にリングバッファにして、シリアル受信バッファと同等に扱えるように変更した。
					XBee用のRAMメモリ容量は今まで以上に必要となったけど、その分シリアル側のバッファを小さくしてもかまわないと思う。
		2011/4/23	APIモードにおける、ATコマンドの送受信環境の整備を本格的に開始
					RSSIを取得する機能を追加し、その完全な動作を確認した。
		2011/5/3	CTSの論理を逆にとっていたのを修正。
					さらに、CTS監視を有効にしている時は、CTS=Highの状態において送信を待つことにした。
		2011/5/5	ver6.2 開発スタート
					ATコマンドの応答しない場合に備えてタイムアウトを設けることとする。
					// #define F_CPU 1000000 // Clock 1MHz
					の様に、クロック周波数の設定があればタイムアウト処理を実行する。
					AVR以外のマイコンで実装する際にはカウントの速度を調整する必要があるかもしれない。
					加えて、ループが単純なので最適化で消えるかも知れない。
		2012/4/13	コメント追加
		2012/12/6	ver6.3と改名して、コメントを修正
		            XBee_hからXBee_INTERRUPT_Hへ改名
                    XBee_buff_check()の返り値を、受信データ数を返すように変更した。
                    つーか、全面改訂。
        2012/12/10  改訂作業をほぼ終える。
                    用語の不統一とかいろいろあるけど、もう疲れたので更新を一時休止します。
        2013/2/9    バッファへ書き込めなければ、Xbee_SetAddressAndPacketSize()を呼び出さないように変更し、また呼び出す際も実際に書き込んだデータ数を渡すように変更した。
        2013/2/23    一部のコメントを追記・訂正
                     XBee_WaitApiPacket()のバグを修正し、任意のパケットを受信するまで待つように変更した。
                     XBee_GetRssi()のタイムアウト時の処理を明確化した。
        2013/3/5     _XBee_ApiParseAndMemory()にて、引き渡しデータサイズを1つ多く間違えていたのでこれを訂正した。
        2013/3/11    コメントアウトの仕方を一部修正した。
		2013/5/4	 コメントアウトを修正
					 Atmel Studio 6.1にて再ビルド後、修正したと思い込んでたバグを修正した。
					 また、Gitによるソースコード管理を行うために、文字コードをUTF8へ書き換えた。
					 C言語における開発はこれで区切りとしたいと思う。
					今後はC++における開発に移行する。
					ところで、最近ArduinoでポピュラーなAPIを読んだんだけど、受信データの処理の仕方はほぼ同じ実装になってた。
					やっぱ同じアルゴリズムになるんだなぁ。

	[6]version:
		2009/12/24	開発開始ver1	
		2010/5/7	ver2.0 色々改造
					バッファとハードのリセットをセットにした関数の整備が必要だ。
					受信パケットのチェックサムの確認も・・
		2010/11/ 6	ver3.0 送信系のプログラムを完成させる
		2010/11/11	ver3.0beta2をリリース
		2010/11/15	ver3.0beta3をリリース
		2010/11/16	ver3.0をリリース
		2011/ 4/14	ver3.1をリリース
		2011/ 4/17	ver4.0をリリース
		2011/ 4/18	ver4.1をリリース
		2011/ 4/22	ver5.0をリリース
		2011/ 4/26	ver6.0をリリース
		2011/ 5/ 3	ver6.1をリリース
		2012/ 4/ 	ver6.2をリリース したつもりでしてなかった。

	tab size：
		4
*****************************************************************************************************************************/
//------XBee header define------------------------------------------------------
#ifndef XBEE_PROTOCOL                                   /*インクルードガード */
#define XBEE_PROTOCOL

//------header file include------------------------------------------------------
#include "string_personal5.5.h"                         /* 自作の文字列処理関数群 */
#include "fifo.h"
#include "XBee.h"
#include "XBeeApiParser.c"
#include <stdbool.h>

//------関数の互換性を保つマクロ--------------			/* ** シリアル関数名に関する変更が生じそうなのはここと、一番下 ** */
#define SCI_PURGE() 			rs_rx_purge()			/* シリアル受信バッファの初期化ルーチン名を指定。シリアル通信プログラムに合わせて変更する必要有り。 */
#define	SCI_getc()				rs_getc()				/* バッファからデータの受信 */
#define SCI_putc(c)				rs_putc(c)				/* 一文字送信 */
#define	SCI_buff_check()		rs_rx_buff()			/* 受信バッファのチェック関数.受信データがあれば、0以外の正の整数を返す. */
//------デバッグ用のマクロ-------------------デバッグ機能を利用しない場合はコメントアウトしてください。 */
//#define XBEE_CONTROL_DEBUG

//------グローバル変数を宣言----------------------------------------------------------
static XBEE_TX_SETTING      _XBeeTxSetting;                 /* 送信設定 */
static XBEE_WS_TX           _XBeeWorkTx;                    /* 送信の処理で利用される */
static XBEE_WS_RX           _XBeeWorkRx;                    /* 受信の処理で利用される */
static char                 _XbeeDummyBuf;                  /* printf()やputs()を利用しない用途向け */
static char                 _XBeePayload[XBee_BUFF_SIZE];	/* 受信データを格納するバッファ */









//------以下は、関数--------------------------------------
/************************************************************************/
/* 生の受信データ数を返す                                               */
/* デバッグ用途を想定しています。                                       */
/************************************************************************/
int XBee_CheckRawBuff()
{
    return SCI_buff_check(); 
}

/************************************************************************/
/* 生の受信データを1 byte返す                                           */
/* デバッグ用途を想定しています。                                       */
/************************************************************************/
char XBee_Raw_getc()
{
    return SCI_getc();
}


/************************************************************************/
/* 受信バッファ・その他を初期化する                                     */
/************************************************************************/
void XBee_rx_purge()
{
    (void)SCI_PURGE();							            // シリアルの受信バッファをクリア
    _XBeeWorkRx.packet.apiId = 0;
    _XBeeWorkRx.payload.rp = 0;
    _XBeeWorkRx.payload.wp = 0;
    _XBeeWorkRx.packet.wp = 0;
    _XBeeWorkRx.packet.ap2MaskFlag = false;
    _XBeeWorkRx.packetCounter = 0;
    _XBeeWorkRx.nodeCounter = 0;
    _XBeeWorkRx.backupAdd.msb = 0ul;
    _XBeeWorkRx.backupAdd.lsb = 0ul;
    _XBeeWorkRx.lock = false;
    _XBeeWorkRx.busy = false;
    _XBeeWorkRx.tws.wp = 0;
    _XBeeWorkRx.tws.overflow = false;
    _XBeeWorkRx.addMask.msb = BROADCAST_ADDRESS_MSB;
    _XBeeWorkRx.addMask.lsb = BROADCAST_ADDRESS_LSB;
    _XBeeWorkTx.checkSum = 0;
    _XBeeWorkTx.frameId = 0x00;
    return;
}

/***********************************************************************************************/
/* 受信マスクをセットする                                                                      */
/*                                                                                             */
/* セットされたアドレス以外のノードからの受信データのみ受信バッファへ保持するようになります。  */
/* 処理を高速にするために、ポインタ渡しを使っています。                                        */
/* 無効にする場合は、ブロードキャストアドレスを入力してください。                              */
/***********************************************************************************************/
void XBee_SetAddMask(XBEE_ADD *add)
{
    _XBeeWorkRx.addMask = *add;
    return;
}

/*************************************************************************/
/* 送受信の方法（API or AT）を設定する                                   */
/*                                                                       */
/* argument:                                                             */
/*      AP: xbee_com_mode　APIモードで通信するか、普通に通信するかを選択 */
/*************************************************************************/
static void _XBee_SetApiMode(char AP)
{
    // 送信設定
    _XBeeTxSetting.apiEnable = AP;
    // 受信ワーキングスペースの初期化
    _XBeeWorkRx.apiEnable = AP;
    return;
}

/******************************************************************************/
/* 送信モードを切り替える                                                     */
/*                                                                            */
/* ブロードキャストとユニキャストの切り替えにご利用ください。                 */
/* 送受信バッファは初期化されません。                                         */
/*                                                                            */
/* argument:                                                                  */
/*          transmit_mode: ユニキャストorブロードキャスト                     */
/******************************************************************************/
void XBee_SetTransmitMode(char transmit_mode)
{
    _XBeeTxSetting.transmitType = transmit_mode;
    return;
}

/******************************************************************************/
/* 送信設定を変更する                                                         */
/*                                                                            */
/* 送信相手を切り替える場合に活用して下さい。                                 */
/* 送受信バッファは初期化されません。                                         */
/*                                                                            */
/* argument:                                                                  */
/*          address_msb  : 送信先上位アドレス                                 */
/*          address_lsb  : 送信先下位アドレス                                 */
/*          network_add  : ZigBeeネットワークアドレス                         */
/*          radius       : ブロードキャスト半径(メッセージをリレーする数)     */
/*          transmit_mode: ユニキャストorブロードキャスト                     */
/******************************************************************************/
void XBee_SetTxSetting(unsigned long int address_msb, unsigned long int address_lsb, unsigned int network_add, char radius, char transmit_mode)
{
    _XBeeTxSetting.add.msb = address_msb;
    _XBeeTxSetting.add.lsb = address_lsb;
    _XBeeTxSetting.networkAddress = network_add;
    _XBeeTxSetting.broadcastRadius = radius;
    _XBeeTxSetting.transmitType = transmit_mode;
    return;
}

/********************************************************************************************************************************************************************/
/* XBeeを使うために初期設定を実施する                                                                                                                               */
/*   argument：                                                                                                                                                     */
/*          address_msb  : 送信先上位アドレス                                                                                                                       */
/*          address_lsb  : 送信先下位アドレス                                                                                                                       */
/*          network_add  : ZigBeeネットワークアドレス                                                                                                               */
/*          radius       : ブロードキャスト半径(メッセージをリレーする数)                                                                                           */
/*          transmit_mode: ユニキャストorブロードキャスト                                                                                                           */
/*          AP           : xbee_com_mode　APIモードで通信するか、普通に通信するかを選択                                                                             */
/*          *tx1         : printf()を使った送信に使用するバッファアドレス　その1（AVRマイコン使用宣言時はputs()も対象になります*）                                  */
/*          tx1_size     : とそのサイズ                                                                                                                             */
/*          *tx2         : printf()を使った送信に使用するバッファアドレス　その2（AVRマイコン使用宣言時以外は不要*）                                                */
/*          tx2_size     : とそのサイズ                                                                                                                             */
/*                                                                                                                                                                  */
/*   *AVRマイコン使用宣言とは、“#define    TARGET_MICON    AVRmicon_HDR”と本ファイルをインクルードする前に宣言していることを指します。                            */
/*                                                                                                                                                                  */
/*   呼び出し例：                                                                                                                                                   */
/*          @AVR:		XBee_init(BROADCAST_ADDRESS_MSB, BROADCAST_ADDRESS_LSB, ?, ?, BROADCAST_TRANSMIT, COM_API_MODE, buff1, sizeof(buff1), buff2, sizeof(buff2)) */
/*          @non AVR:	XBee_init(BROADCAST_ADDRESS_MSB, BROADCAST_ADDRESS_LSB, ?, ?, BROADCAST_TRANSMIT, COM_API_MODE, buff1, sizeof(buff1), buff1, sizeof(buff1)) */
/*          上記の?には、適当なネットワークアドレスとブロードキャスト半径を格納して下さい。        	                                                                */
/********************************************************************************************************************************************************************/
void XBee_init(unsigned long int address_msb, unsigned long int address_lsb, unsigned int network_add, char radius, char transmit_mode, char AP, char *tx1, int tx1_size, char *tx2, int tx2_size)
{
    // 送信設定
    (void)XBee_SetTxSetting(address_msb, address_lsb, network_add, radius, transmit_mode);
    // 送信ワーキングスペースの初期化
    _XBeeWorkTx.buf1.buff = tx1;            /* 引数指定がなければ省略可能 <- 文字コードの関係？で、//を使うとエラーが発生する @ AtmelStudio 6.0。*/
    _XBeeWorkTx.buf1.len = tx1_size;        /* 引数指定がなければ省略可能 */
    _XBeeWorkTx.buf2.buff = tx2;            /* 引数指定がなければ省略可能 */
    _XBeeWorkTx.buf2.len = tx2_size;        /* 引数指定がなければ省略可能 */
    // 受信ワーキングスペースの初期化
    _XBeeWorkRx.payload.buff = _XBeePayload;
    _XBeeWorkRx.payload.len = sizeof(_XBeePayload);
    // その他
    (void)XBee_rx_purge();
    (void)_XBee_SetApiMode(AP);
    _XbeeDummyBuf = 0x00;                   // 警告抑制
    // ノード管理関係
    #ifdef XBEE_NODE_MANAGER_C
        (void)Xbee_InitManager();
    #endif
}

/********************************************************************************************************************************************************************/
/* XBeeを使うために初期設定を実施する                                                                                                                               */
/*                                                                                                                                                                  */
/* printf()系の関数呼び出しを伴わない場合の初期化です。                                                                                                             */
/* 絶対にprintf()系を呼び出さないようにして下さい。                                                                                                                 */
/* また、AVRマイコンで省RAM設定を行った場合はputs()系も呼び出さないようにして下さい。                                                                               */
/*   argument：                                                                                                                                                     */
/*          address_msb  : 送信先上位アドレス                                                                                                                       */
/*          address_lsb  : 送信先下位アドレス                                                                                                                       */
/*          network_add  : ZigBeeネットワークアドレス                                                                                                               */
/*          radius       : ブロードキャスト半径(メッセージをリレーする数)                                                                                           */
/*          transmit_mode: ユニキャストorブロードキャスト                                                                                                           */
/*          AP           : xbee_com_mode　APIモードで通信するか、普通に通信するかを選択                                                                             */
/*                                                                                                                                                                  */
/*   呼び出し例：                                                                                                                                                   */
/*          @AVR:		XBee_init(BROADCAST_ADDRESS_MSB, BROADCAST_ADDRESS_LSB, ?, ?, BROADCAST_TRANSMIT, COM_API_MODE)                                             */
/*          @non AVR:	XBee_init(BROADCAST_ADDRESS_MSB, BROADCAST_ADDRESS_LSB, ?, ?, BROADCAST_TRANSMIT, COM_API_MODE)                                             */
/*          上記の?には、適当なネットワークアドレスとブロードキャスト半径を格納して下さい。        	                                                                */
/********************************************************************************************************************************************************************/
void XBee_init2(unsigned long int address_msb, unsigned long int address_lsb, unsigned int network_add, char radius, char transmit_mode, char AP)
{
    (void)XBee_init(address_msb, address_lsb, network_add, radius, transmit_mode, AP, &_XbeeDummyBuf, 1, &_XbeeDummyBuf, 1);
    return;
}








/**************************** 以下は送信系他 *********************************/

/***********************************************************************************/
/* 返信の取得を認識するために、前回の通信に使用したフレームIDとは異なるIDを返す    */
/* フレームIDの決定を本関数に頼る限り、ひとつ前のIDとは異なることを保証します。    */
/***********************************************************************************/
static char _XBee_GetFrameId(void)
{
    char frame_id;
    
    frame_id = (char)(((int)_XBeeWorkTx.frameId + 1) % 128);
    if(frame_id == 0x00)frame_id++;                             // frame_idが0だと、応答が返ってこない。
    _XBeeWorkTx.frameId = frame_id;
    
    return frame_id;
}

/************************************************************************/
/* 引数で受け取った１文字だけを送信する（シリアルポートへスルーパス）   */
/*                                                                      */
/* シリアルポートへデータの送信を依頼しています。                       */
/* 本ソースコード中からの呼び出しのみを想定している。                   */
/* 外部からこの関数を呼び出しても、APIモードならば無線伝送されません。  */
/* XBeeのCTSポートが定義されていれば、その状況を確認して送信します。    */
/*                                                                      */
/* 引数: c      : 送信したいデータ                                      */
/************************************************************************/
static void _sci_putc(char c)
{
    #ifndef XBEE_TX_DATA_CREATOR_H_
	    #ifdef XBee_CTS_IO							// CTSポート監視用のマイコン側ポートが#defineで定義されていれば有効
		    while(XBee_CTS_IO);						// XBeeがCTSをHIGHにしている間は出力を抑制する。無視すると、送信ミスが多発する。
		    (void)SCI_putc(c);
	    #else
		    (void)SCI_putc(c);
	    #endif
    #else
        (void)Xbee_PutcDummy(c);
    #endif
	return;
}

/************************************************************************/
/* 引数で受け取った１文字だけを送信する                                 */
/*                                                                      */
/* 本ソースコード中からの呼び出しのみを想定している。                   */
/* 外部からこの関数を呼び出しても、APIモードならば無線伝送されません。  */
/* API通信に備えて、チェックサムの計算も実施しています。                */
/* また、AP == 2の設定時の対応を自動的にとります。                      */
/*                                                                      */
/* 引数: c      : 送信したいデータ                                      */
/************************************************************************/
static void _XBee_putc(char c)
{
    _XBeeWorkTx.checkSum += (int)(c);                       // チェックサムの計算
    _XBeeWorkTx.checkSum = _XBeeWorkTx.checkSum & 0x00ff;
    if(_XBeeTxSetting.apiEnable == API_WITH_ESCAPES)        // AP == 2の時の処理
    {
        if(c == 0x7e || c == 0x7d || c == 0x11 || c == 0x13)
        {
            (void)_sci_putc(START_ESCAPE_CHAR);
            (void)_sci_putc(0x20 ^ c);
        }
        else
            (void)_sci_putc(c);
    }
    else
        (void)_sci_putc(c);
    return;
}

/************************************************************************/
/* 指定数だけ文字列を送信する                                           */
/*                                                                      */
/* 本ソースコード中からの呼び出しのみを想定している。                   */
/* 外部からこの関数を呼び出しても、APIモードならば無線伝送されません。  */
/*                                                                      */
/* 引数: *buff  : 送信したいデータを格納した配列の先頭アドレス          */
/*       size   : 送信したいデータ数                                    */
/************************************************************************/
static void _XBee_putsb(char *buff, int size)
{
	int  i = 0;
	char c;

	do
    {
		c = *(buff + i);
		_XBee_putc(c);
		if(_XBeeTxSetting.apiEnable == API_OFF && c == '\n')    // ATコマンドモード時には改行コードへ対応する
			_XBee_putc('\r');
		i++;
	}while(--size);

	return;
}

/************************************************************************/
/* 渡されたバッファのデータをAPIモードで送信する                        */
/*                                                                      */
/* 本ソースコード中からの呼び出しのみを想定している。                   */
/* 1回のパケットサイズはmax72                                           */
/* argment: *buff   : 送信したいバッファの先頭アドレス                  */
/*          size    : 送信したい文字数（データサイズ）                  */
/*          add_msb : 送信先アドレスの上位32ビット                      */
/*          add_lsb : 送信先アドレスの下位32ビット                      */
/************************************************************************/
static void _XBee_API_tx(char *buff, int size, unsigned long int add_msb, unsigned long int add_lsb)
{
    int  i, length, rp = 0;
    char frame_id;

    do
    {
        if(size > XBee_API_PACKET_MAX_SIZE_FOR_TX)                 // 出力サイズの決定
        {
            length = XBee_API_PACKET_MAX_SIZE_FOR_TX;
            size -= XBee_API_PACKET_MAX_SIZE_FOR_TX;               // 未出力データサイズを計算
        }
        else
        {
            length = size;
            size = 0;
        }

        (void)_sci_putc(START_DELIMITER_CHAR);              // APIのフレームデリミタを送信。直接送信すれば、AP==2の面倒さを回避できる。
        // API Identifier(1byte) + Frame ID(1byte) + 64bit address(8byte) + network address(2byte) + Broadcast Radius(1byte) + Options(1byte) + RF data(size byte)
        _XBee_putc((char)(((length + 14) >> 8) & 0xff));    // length 上位
        _XBee_putc((char)((length + 14) & 0xff));           // length 下位
        _XBeeWorkTx.checkSum = 0;                           // チェックサムの初期化
        _XBee_putc(0x10);                                   // API ID
        frame_id = _XBee_GetFrameId();
        _XBee_putc(frame_id);                               // Frame ID
        								
        i = 4;
        while(i--)_XBee_putc((char)(add_msb >> (i * 8)));	// 64bit addressを送信
        i = 4;
        while(i--)_XBee_putc((char)(add_lsb >> (i * 8)));		
        //XBee_putsb(add, 8);
        _XBee_putc((char)((_XBeeTxSetting.networkAddress >> 8) & 0xff));// network address, 2011/2/14リリースのS1用データシートによると予備とされている。ZB(2010/11/15リリース)ではネットワークアドレスとされている。
        _XBee_putc((char)(_XBeeTxSetting.networkAddress & 0xff));
        _XBee_putc(_XBeeTxSetting.broadcastRadius);         // Broadcast Radius
        _XBee_putc(0x00);                                   // Options. bit0:disable ACK, bit1:Don't attempt route Discovery, All other bit must be set 0.
        _XBee_putsb(buff + rp, length);                     // buffの中身を出力する
        rp += length;                                       // 次回の出力アドレスを更新
        _XBee_putc((char)(0xff - _XBeeWorkTx.checkSum));    // Checksum送信
    }while(size);                                           // データが残っていれば、繰り返す

    return;
}

/************************************************************************/
/* 渡されたバッファのデータを送信する                                   */
/*                                                                      */
/* 送信設定によらず、指定したアドレスへデータを送信します。             */
/* argment: *buff   : 送信したいバッファの先頭アドレス                  */
/*          size    : 送信したい文字数（データサイズ）                  */
/*          add_msb : 送信先アドレスの上位32ビット                      */
/*          add_lsb : 送信先アドレスの下位32ビット                      */
/************************************************************************/
void XBee_send1(char *buff, int size, unsigned long int add_msb, unsigned long int add_lsb)
{
    if(_XBeeTxSetting.apiEnable == API_WITHOUT_ESCAPES || _XBeeTxSetting.apiEnable == API_WITH_ESCAPES)
        (void)_XBee_API_tx(buff, size, add_msb, add_lsb);
    else
        (void)_XBee_putsb(buff, size);           /* そのまま出力 */
    return;
}
/************************************************************************/
/* 渡されたバッファのデータを送信する                                   */
/*                                                                      */
/* 指定されたデータサイズだけ、送信します。                             */
/* 送信先は送信設定に従います。                                         */
/* argment: *buff   : 送信したいバッファの先頭アドレス                  */
/*          size    : 送信したい文字数（データサイズ）                  */
/************************************************************************/
void XBee_send2(char *buff, int size)
{
    if(_XBeeTxSetting.apiEnable == API_WITHOUT_ESCAPES || _XBeeTxSetting.apiEnable == API_WITH_ESCAPES) /* API通信モードであれば */
    {
        if(_XBeeTxSetting.transmitType == UNICAST_TRANSMIT)
            (void)_XBee_API_tx(buff, size, _XBeeTxSetting.add.msb, _XBeeTxSetting.add.lsb);             /* ユニキャストで送信する */
        else
            (void)_XBee_API_tx(buff, size, BROADCAST_ADDRESS_MSB, BROADCAST_ADDRESS_LSB);               /* ブロードキャストで送信する */
    }
    else                                                                                                /* ATコマンドモード（透過モード）と見なす */
        (void)_XBee_putsb(buff, size);                                                                  /* そのまま出力 */
    return;
}
/************************************************************************/
/* 渡されたバッファのデータを送信する                                   */
/*                                                                      */
/* ヌル文字までのデータを送信します。                                   */
/* 送信先はXBee構造体の情報に従って決定されます。                       */
/* argment: *buff   : 送信したいバッファの先頭アドレス                  */
/************************************************************************/
void XBee_send3(char *buff)
{
    int i = 0;

    while(*(buff + i)) i++;                      /* ヌルを除いた文字数をカウント */
    (void)XBee_send2(buff, i);
	return;
}

/************************************************************************/
/* 一文字だけ送信する                                                   */
/************************************************************************/
void XBee_sendc(char c)
{
	XBee_send2(&c, 1);
	return;
}

/****************************************************************************************/
/* APIモードで任意のATコマンドを送信する                                                */
/*                                                                                      */
/* XBeeハードウェア設定などを行います。                                                 */
/* argument:                                                                            */
/*          frame_ID: 通信に使用するフレームID（コマンドの応答を識別するのに使えます）  */
/*          commandName_H: コマンド識別子の上位                                         */
/*          commandName_L: コマンド識別子の下位                                         */
/*          param      : 送信するパラメータ                                             */
/*          size       : パラメータのサイズ                                             */
/* 2011/4/23 完成                                                                       */
/****************************************************************************************/
void XBee_SendAtCommand(char frame_ID, char commandName_H, char commandName_L, char *param, int size)
{
    int i;

    if(_XBeeTxSetting.apiEnable == API_WITHOUT_ESCAPES || _XBeeTxSetting.apiEnable == API_WITH_ESCAPES)
    {
        (void)_sci_putc(START_DELIMITER_CHAR);              // APIのフレームデリミタを送信。
        (void)_sci_putc(0x00);
        (void)_sci_putc((char)((4 + size) & 0xff));         // lengthの送信
        _XBeeWorkTx.checkSum = 0;                           // チェックサムの初期化
        (void)_XBee_putc(AT_COMMAND);					    // frame type
        (void)_XBee_putc(frame_ID);						    // frame_ID: ACKの識別に使用するので、好きに設定してよい
        (void)_XBee_putc(commandName_H);
        (void)_XBee_putc(commandName_L);
        i = 0;
        while(i < size)(void)_XBee_putc(*(param + i++));    // Parameter
        _XBee_putc((char)(0xff - _XBeeWorkTx.checkSum));    // Checksum送信
    }
    return;
}
























/**************************** 以下は受信系他 *********************************/
/********************************************************************************/
/* 一次解析済みのAPIパケットをさらにパケットの種類毎に解析し、結果を保存します。*/
/*                                                                              */
/* 結果は引数に格納します。                                                     */
/* lockビットがtrueならば、処理を行いません。                                   */
/* また、必要ならノードとパケット管理関数を呼び出します。                       */
/* argument:  *wsrx: 受信データを保持する構造体へのポインタ                     */
/********************************************************************************/
static void _XBee_ApiParseAndMemory(XBEE_WS_RX *wsrx)
{
    int  i, t;

    wsrx->packetCounter = (wsrx->packetCounter + 1) % 30000;// 新しいパケットが受信されたことをこの数字で把握する。
    if(wsrx->lock == false)
    {
        switch(wsrx->packet.apiId)
        {
            case REMOTE_AT_COMMAND_RQ:              // Remote AT Command Request ・・これは必要なのか?
                break;
            case AT_COMMAND_RESPONSE:               // AT Command Response
                (void)XBee_ParseApiATCommandResponse(wsrx->packet.buff, &wsrx->atcr.acr);
                break;
            case TRANSMIT_STATUS:                   // Transmit Status
                (void)XBee_ParseTransmitStatus(wsrx->packet.buff, &wsrx->ts);
                break;
            case RX_INDICATOR:                      // @ AO == 0 ,パケット受信処理。
                (void)XBee_ParseApiReceivePacket(wsrx->packet.buff, &wsrx->rdp);
                // 処理対象のノードからのデータであれば、各種処理を実行する
                if ((_XBeeWorkRx.addMask.msb == BROADCAST_ADDRESS_MSB && _XBeeWorkRx.addMask.lsb == BROADCAST_ADDRESS_LSB) || 
                    (_XBeeWorkRx.addMask.msb == wsrx->rdp.add.msb && _XBeeWorkRx.addMask.lsb == wsrx->rdp.add.lsb))
                {
                    PA.PORT.BIT.B7 ^= 1;
                    //rs2_puts("g\n");
                    /*
                    #ifdef XBEE_CONTROL_DEBUG
                        XBee_send3("db: ");
                        {
                            char hoge[10];
                            (void)Unti_atoi(hoge, sizeof(hoge), (long)wsrx->rdp.size);
                            XBee_send3(hoge);                       
                        }
                        XBee_send3("\n");
                    
                        //rs2_printf("size: %d\n", wsrx->rdp.size);
                        //rs2_puts("payload: ");
                        //rs2_putsb((char *)wsrx->rdp.dataPacket, wsrx->rdp.size);
                        //rs2_puts("\n");
                    #endif
                    */
                    // 受信したバッファの内容を別のバッファへコピー
                    for (i = 0; i < wsrx->rdp.size; i++)                        // 最終的にiが代入した文字数を表す。
                    {
                        t = (wsrx->payload.wp + 1) % XBee_BUFF_SIZE;
                        wsrx->payload.buff[wsrx->payload.wp] = wsrx->rdp.dataPacket[i];
                        if(t != wsrx->payload.rp)
                            wsrx->payload.wp = t;                               // 書き込みがバッファの読み出しに追いつくと、残りは無視。
                        else
                            break;
                    }
                    // あるノードからの初回の受信・もしくは前回と異なるノードから受信したことを識別
                    if (wsrx->rdp.add.msb != wsrx->backupAdd.msb || wsrx->rdp.add.lsb != wsrx->backupAdd.lsb)
                    {
                        wsrx->nodeCounter = (wsrx->nodeCounter + 1) % 30000;    // この数字変わっていれば、前回と異なるノードからの受信であると分かる。
                        wsrx->backupAdd = wsrx->rdp.add;
                    }
					// パケットとノード管理が必要とされるなら、関数を呼び出して情報を伝える
                    #ifdef XBEE_NODE_MANAGER_C
                        if (i > 0)
                            (void)Xbee_SetAddressAndPacketSize(&wsrx->rdp.add, i);
                    #endif
                }
                break;
            case EXPLICIT_RX_INDICATOR:                                         // Explicit Rx Indicator
                break;
            default:
                break;
        }
    }    
    return;
}
/*****************************************************************************************************/
/* 渡されたデータを数珠つなぎにして受信データを処理する                                              */
/*                                                                                                   */
/* シリアルの受信割り込みからの直接呼出し / 手動処理_XBee_RxNoInterrupt()での使用を想定している。    */
/* 処理結果は本ソースコード内に宣言している外部変数のXBee内に保存されます。                          */
/*****************************************************************************************************/
void XBee_RxInterrupt(char c)
{
	int t;
    char apiId = 0x00, c2;

    _XBeeWorkRx.busy = true;                            // この行はタイマー割り込みでどうのこうのとない限り、意味ないなぁ。
    if(_XBeeWorkRx.apiEnable == API_WITHOUT_ESCAPES || _XBeeWorkRx.apiEnable == API_WITH_ESCAPES)
    {
        if(_XBeeWorkRx.lock == false)
        {
            // まずは、退避バッファにデータがあればそれをひとまず処理する。
            for(t = 0; t < _XBeeWorkRx.tws.wp; t++)
            {
                c2 = _XBeeWorkRx.tws.buff[t];
                apiId = XBee_ParseApiPacketc(c2, &_XBeeWorkRx.packet, _XBeeWorkRx.apiEnable);
                if(apiId != 0x00 && apiId != 0xff)
                    (void)_XBee_ApiParseAndMemory(&_XBeeWorkRx);
            }
            // 受信したてのデータを解析に回す
            _XBeeWorkRx.tws.wp = 0;
            apiId = XBee_ParseApiPacketc(c, &_XBeeWorkRx.packet, _XBeeWorkRx.apiEnable);
            if(apiId != 0x00 && apiId != 0xff) 
                (void)_XBee_ApiParseAndMemory(&_XBeeWorkRx);
        }
        else  // もし、都合により処理が一時的に停止させられていた場合（割り込み処理でしか意味がないが、そもそも多重割り込みを禁止しているのだから意味ないんじゃないの？）
        {
            if(_XBeeWorkRx.tws.wp < (int)sizeof(_XBeeWorkRx.tws.buff))  // 可能なら一次退避バッファへ保存する
                _XBeeWorkRx.tws.buff[_XBeeWorkRx.tws.wp++] = c;
            else
            {
                _XBeeWorkRx.tws.wp = 0;
                _XBeeWorkRx.tws.overflow = true;
            }                
        }
    }
    else
    {
        t = (_XBeeWorkRx.payload.wp + 1) % XBee_BUFF_SIZE;
        _XBeeWorkRx.payload.buff[_XBeeWorkRx.payload.wp] = c;
        if(t != _XBeeWorkRx.payload.rp) 
            _XBeeWorkRx.payload.wp = t;         // 割り込み受信データは次々とくるので、書き込みがバッファの読み出しに追いつくと、同じ点に新受信データが書き込む。
    }
    _XBeeWorkRx.busy = false;
    return;
}
/*****************************************************************************************************/
/* シリアルバッファから受信データを受け取って処理する                                                */
/*   非割り込み処理用に整備しました。                                                                */
/*                                                                                                   */
/* 未処理のデータがシリアルバッファ内に残っている場合は全て処理します。                              */
/* 未処理データの処理が終わると、returnします。                                                      */
/* 割り込み処理を実施している場合は、スルーされます。                                                */
/* 処理結果は本ソースコード内に宣言している外部変数のXBee内に保存されます。                          */
/*****************************************************************************************************/
static void _XBee_RxNoInterrupt(void)
{
    #ifndef XBee_INTERRUPT_H                            // これがあると、割り込み処理とみなす
        while(SCI_buff_check())
        {
            (void)XBee_RxInterrupt(SCI_getc());
        }
    #endif

    return;
}

/************************************************************************/
/* バッファをチェックし、受信データサイズを返す                         */
/*                                                                      */
/* APIモードで非割り込みならば受信処理&解析を実施する                   */
/* 返値：受信データサイズ                                               */
/*                                                                      */
/* 2010/11/15	新規作成                                                */
/************************************************************************/
int XBee_buff_check(void)
{
    int     size, ans = 0;

    #ifndef XBee_INTERRUPT_H                            // 無くても変わらないが、非割り込み処理時用の記述であることを明示しておく
        (void)_XBee_RxNoInterrupt();
    #endif

    size = _XBeeWorkRx.payload.wp - _XBeeWorkRx.payload.rp;
    if(size < 0)
        size += XBee_BUFF_SIZE;
    ans = size;

    return ans;
}

/************************************************************************/
/* 受信した文字を一文字返す                                             */
/*                                                                      */
/* return value: 受信した文字を一つ(バッファが空ならヌルを返す)         */
/************************************************************************/
char XBee_getc(void)
{
    char ans = 0;

    if(XBee_buff_check())
    {
        ans = _XBeeWorkRx.payload.buff[_XBeeWorkRx.payload.rp++];       // もし受信データがあれば1文字引き出す
        _XBeeWorkRx.payload.rp %= XBee_BUFF_SIZE;
        #ifdef XBEE_NODE_MANAGER_C                                      // パケットとノード管理が必要とされるなら、関数を呼び出して情報を伝える
            (void)Xbee_GivePullOutNum(1);
        #endif
    }
    return ans;
}

/************************************************************************/
/* 受信データを返す                                                     */
/*                                                                      */
/* 受信した内容は引数で渡したバッファへ格納されます。                   */
/* 受信データはバイナリの可能性もあります。                             */
/* バッファオーバーランを検出しないので注意して下さい。                 */
/* argument    : 結果を格納したいバッファの先頭アドレス                 */
/* return value: 格納したデータ数                                       */
/************************************************************************/
int XBee_gets(char *buff)
{
    int i = 0;

    while(XBee_buff_check())
    {
        *(buff + i++) = _XBeeWorkRx.payload.buff[_XBeeWorkRx.payload.rp++]; // もし受信していたらデータ列を引き出す
        _XBeeWorkRx.payload.rp %= XBee_BUFF_SIZE;
    }
    #ifdef XBEE_NODE_MANAGER_C                                              // パケットとノード管理が必要とされるなら、関数を呼び出して情報を伝える
        (void)Xbee_GivePullOutNum(i);
    #endif
    return i;
}
/************************************************************************/
/* 受信データを指定データ数内で返す                                     */
/*                                                                      */
/* 受信した内容は引数で渡したバッファへ格納されます。                   */
/* 受信データはバイナリの可能性もあります。                             */
/* argument    : buff: 結果を格納したいバッファの先頭アドレス           */
/*               size: buffのサイズ                                     */
/* return value: 格納したデータ数                                       */
/************************************************************************/
int XBee_getsb(char *buff, int size)
{
    int i = 0;

    while(XBee_buff_check() && i < size)
    {
        *(buff + i++) = _XBeeWorkRx.payload.buff[_XBeeWorkRx.payload.rp++]; // もし受信していたらデータ列を引き出す
        _XBeeWorkRx.payload.rp %= XBee_BUFF_SIZE;
    }
    #ifdef XBEE_NODE_MANAGER_C                                              // パケットとノード管理が必要とされるなら、関数を呼び出して情報を伝える
        (void)Xbee_GivePullOutNum(i);
    #endif
    return i;
}









/**************************** 以下はATコマンド関係 *********************************/

/*****************************************************************************/
/* XBeeの受信信号強度（RSSI）を返す                                          */
/*                                                                           */
/* API通信モードでなければ何もせずに処理を戻します。                         */
/* クロック周波数[Hz]としてF_CPUが定義されていればタイムアウトが動作します。 */
/* return value: RSSI[-dBm], タイムアウト時：0xff                            */
/*****************************************************************************/
char XBee_GetRssi(void)
{
	char *dummy = 0;
	char frame_id, RSSI = 0;
	#ifdef F_CPU
		long int i = 0l;
	#endif
	
    if (_XBeeWorkRx.apiEnable == API_WITHOUT_ESCAPES || _XBeeWorkRx.apiEnable == API_WITH_ESCAPES)
    {
	    frame_id = _XBee_GetFrameId();
        // この辺は、受信処理の設定となる	
	    _XBeeWorkRx.lock = false;                               // 受信データの更新を許可
	    _XBeeWorkRx.atcr.timeOut = false;                       // タイムアウトフラグをクリア
	    (void)XBee_SendAtCommand(frame_id, 'D', 'B', dummy, 0); // RSSIの送信要求
	    while(_XBeeWorkRx.atcr.acr.frameId != frame_id)         // 受信するまで待つ
        {
            #ifndef XBee_INTERRUPT_H                            // 無くても変わらないが、非割り込み処理時用の記述であることを明示しておく
                (void)_XBee_RxNoInterrupt();
            #endif
		    #ifdef F_CPU
			    i += 60;
			    if(i > (long int)F_CPU)
                {
				    _XBeeWorkRx.atcr.timeOut = true;
				    break;                                      // time out
			    }
		    #endif
	    }
        if(_XBeeWorkRx.atcr.timeOut == false)
        {
	        _XBeeWorkRx.lock = true;                            // 受信データの更新を不許可
	        RSSI = *_XBeeWorkRx.atcr.acr.param;                 // 受信データを取得する
            _XBeeWorkRx.lock = false;                           // 受信データの更新を許可
        }
        else
            RSSI = 0xff;                                        // タイムアウト時の対応
    }
	return RSSI;
}












/**************************** 以下はその他 *********************************/

/************************************************************************/
/* ATコマンドレスポンスの処理結果を返す                                 */
/* return value: ATコマンドレスポンスの処理結果                         */
/************************************************************************/
XBEE_ACRRRS XBee_GetAtcResult(void)
{
    XBEE_ACRRRS ans;
    
    _XBeeWorkRx.lock = true;
    ans = _XBeeWorkRx.atcr;
    _XBeeWorkRx.lock = false;
    return ans;
}

/********************************************************************************************************************************************************/
/* 直前の送信の結果を返す                                                                                                                               */
/* ただし、本関数実行時には未だ送信結果を受信していない可能性もあります。                                                                               */
/* 従って確実に送信状況を把握したい場合は、データ送信直後にXBee_WaitApiPacket(TRANSMIT_STATUS)を使って送信結果が返るのを待つようにして下さい。          */
/* 待った後で本関数を呼び出せば間違いありません。                                                                                                       */
/* return value: 送信状況                                                                                                                               */
/********************************************************************************************************************************************************/
XBEE_TS XBee_GetTransmitStatus(void)
{
    XBEE_TS ans;
    
    _XBeeWorkRx.lock = true;
    ans = _XBeeWorkRx.ts;
    _XBeeWorkRx.lock = false;
    return ans;
}

/************************************************************************/
/* APIパケットの受信カウンター（30000回でリセット）のカウント値を返す   */
/* パケットの受信確認にどうぞ                                           */
/************************************************************************/
int XBee_GetPacketCounter(void)
{
    return _XBeeWorkRx.packetCounter;
}

/*****************************************************************************/
/* 指定したAPIパケットを新たに受信するまで待つ                               */
/*                                                                           */
/* API通信モードでなければ何もせずに処理を戻します。                         */
/* クロック周波数[Hz]としてF_CPUが定義されていればタイムアウトが動作します。 */
/* return value: 0: 受信した，1: タイムアウト                                */
/*****************************************************************************/
char XBee_WaitApiPacket(char apiId)
{
    char timeOut = 0;
    //int count;
    #ifdef F_CPU
        long int i = 0l;
        long int limit = (long int)F_CPU * 10l;
    #endif
    
    //count = XBee_GetPacketCounter();
    _XBeeWorkRx.packet.apiId = 0x00;
    while(_XBeeWorkRx.packet.apiId != apiId)
    {
        #ifndef XBee_INTERRUPT_H                            // 無くても変わらないが、非割り込み処理時用の記述であることを明示しておく
            (void)_XBee_RxNoInterrupt();
        #endif
        #ifdef F_CPU
            i += 1l;
            if(i > limit)
            {
                timeOut = 1;
                break;
            }
        #endif
    }
    return timeOut;
}

/************************************************************************/
/* ノードの受信カウンター（30000回でリセット）のカウント値を返す        */
/* 異なるノードからの信号を受信するたびに更新される値を返します。       */
/* ノードの受信確認にどうぞ                                             */
/************************************************************************/
int XBee_GetNodeCounter(void)
{
    return _XBeeWorkRx.nodeCounter;
}

/***************************************************************************/
/* 直前に受信したRFデータの送信元ノードのアドレスとRSSIを返す              */
/***************************************************************************/
XBEE_NODE XBee_GetNodeAddAndRssi(void)
{
    XBEE_NODE ans;

    ans.Rssi = XBee_GetRssi();              // RSSIとアドレスの取得順が逆だと、直前のデータに対するRSSIであることの保証ができない。
    ans.add = _XBeeWorkRx.rdp.add;          // RSSIの通知命令を送信して応答を待つ前に、とあるノードからのデータを受信し得るので、RSSIを取得後にアドレスを取れば間違いなく直前に受信したデータのRSSIを得られる。
    
    return ans;
}











/**************************** 以下は関数の置換関係 *********************************/
/**********************************************
 sci関係関数を置換

 	rs_putc()等の関数名は、このプログラムを使用する各人で調整してください。
	AVRで使う場合、PSTRに文字数制限を加えられない（自分でマクロを作ればいいのか?）ので、バッファのサイズに注意
**********************************************/


#define rs_putc(c)              XBee_sendc(c)                       // 1文字送信
#define rs_putsb(buff, size)    XBee_send2(buff, size)              // 送信量指定（バイナリなど）の送信


#ifdef TARGET_MICON
	#if TARGET_MICON == AVRmicon_HDR                                // AVRマイコンがターゲットなら、RAMを消費しないように定義（若干送信が遅れるが、クロックが早いなら気にならないでしょう）
		#include<avr/pgmspace.h>                                    // プログラム領域のメモリにアクセスするために必要

		//このヘッダ以降の関数では、プログラムメモリを積極的に利用するように変更する
		#ifndef rs_printf
			#define rs_printf(str, ...)		XBee_send2(mo_sprintf_s(_XBeeWorkTx.buf1.buff, _XBeeWorkTx.buf1.len, strcpy_P(_XBeeWorkTx.buf2.buff, PSTR(str)), __VA_ARGS__), 0)	// 予め文字列処理をした後に呼び出すようにする。
		#endif
		#ifndef rs_puts
			#define rs_puts(str)			XBee_send3(strcpy_P(_XBeeWorkTx.buf1.buff, PSTR(str)))
		#endif
	#endif
#endif

#ifndef rs_printf                                                   // デフォルト（例えばH8）ならば、以下を定義
    #define rs_printf(str, ...)				XBee_send2(mo_sprintf_s(_XBeeWorkTx.buf1.buff, _XBeeWorkTx.buf1.len, str, __VA_ARGS__), 0)	// 予め文字列処理をした後に呼び出すようにする。
#endif
#ifndef rs_puts
    #define rs_puts(buff)					XBee_send3(buff)
#endif

#define rs_rx_buff()                        XBee_buff_check()       // バッファのチェックをXBee関係に置換
#define	rs_getc()                           XBee_getc()             // 1文字受信を置換
#define	rs_gets(buff)                       XBee_gets(buff)         // 文字列受信を置換
#define	rs_getsb(buff, size)                XBee_getsb(buff, size)  // 文字列受信を置換
#define rs_rx_purge()                       XBee_rx_purge()         // 受信バッファの初期化


#endif // >#ifndef XBEE_PROTOCOL
