﻿/************************************************************
XBeeの処理を割り込みを通じて処理させるためのヘッダファイル

注意：
	シリアル通信ヘッダよりも上位でインクルードして下さい。

履歴：
	2012/11/6   コメントを追加
	            XBee_hからXBee_INTERRUPT_Hへ改名
	            インクルードガードを追加
    2012/11/8   コメントを追加
                プロトタイプ宣言を必要な分だけに絞る
    2013/2/23
************************************************************/
#ifndef XBee_INTERRUPT_H		// インクルードガード
#define XBee_INTERRUPT_H

//------header file include------------------------------------------------------
#include "XBee.h"
#include <stdbool.h>

/***********************************************************************************************/
/* 割り込みで受信データを処理するための記述                                                    */
/*                                                                                             */
/* シリアル受信割り込み関数内にSCI_RX_INTERRUPTと書いてください。                              */
/* 既にSCI_RX_INTERRUPTが宣言されているようだと、コンパイル時にコンソールへエラーを表示します。*/
/*                                                                                             */
/* 記述例（AVRマイコンの場合）：                                                               */
/*
SIGNAL (USART1_RX_vect)
{
    char c;
    volatile short t;

    SCI_1.UCSRA.BYTE &= 0xc7;
    c = SCI_1.UDR;
    t = (SCI2.rx.wp + 1) % SCI2.rx.len;
    SCI2.rx.buff[SCI2.rx.wp] = c;
    if (t != SCI2.rx.rp) SCI2.rx.wp = t;

    #ifdef SCI2_RX_INTERRUPT
    SCI2_RX_INTERRUPT
    #endif
}
*/
/* 重要なのは、受信したデータが変数cに格納されるということと、下記の「#define SCI_RX_INTERRUPT (void)XBee_RxInterrupt(c);」で変数cが渡されているとことです。*/
/* 変数名が異なる場合は動作しません。                                                            */
/* また、#ifdefで囲まれた「SCI2_RX_INTERRUPT」がなければやはり動作しません。                     */
/* 必ず記述するようにして下さい。                                                                */
/*************************************************************************************************/
#ifndef SCI_RX_INTERRUPT
	#define SCI_RX_INTERRUPT (void)XBee_RxInterrupt(c);                              // 割り込み関数内で、受信した文字を変数cへ格納する必要がある
#else
	#error "XBee include error. SCI_RX_INTERRUPT has already been defined."
#endif

// プロトタイプ宣言
void XBee_RxInterrupt(char c);

#endif

