﻿/*----------------------------------------------------------------------
「文字列処理関数」
平成19年6月25日 八代工業高等専門学校 生産情報工学専攻2年
					モリシタカツヒロ（森下 功啓）製作

  最終更新：2009/9/11
  更新履歴：
    2007/6/25   
	2009/6/12	熊大院2年になった。…おっさんだ。ま、それは良いとして、今日は関数を追加して、ver2.0へ上げた。
	2009/9/11	特定文字の計測ルーチンを追加。エラーコードを#defineで宣言しなおす。
	2009/9/24	左詰のUnti_atoiを宣言（昔作っていたような気もするが・・・）
	2009/11/21	ver3.0へアップ.一部をvolatile宣言に合わせてちょっと改造
    2009/11/23  アスキーコードの定数宣言をdefine文でここに宣言する
	2010/1/12	defineに"水平タブ"を追加
	2010/2/19	defineの構造を変更して、可読性を上げる。
	2010/2/26	自作のsprintfを追加。
	2010/2/27	sprintfの返り値を文字列のポインタに変更
	2010/4/4	defineでヌル文字を追加
	2010/5/2	string_personalへ改名
	2010/5/31	sprintfのプロトタイプを追加
	2010/7/20	ver5.2 sprintfの一部修正
	2010/8/28	ver5.3 Unti_atoiを修正
	2010/11/11	mo_printfをヌルで閉める様に変更
	2010/11/15	ver5.4 mo_sprintf（C言語で云うspritnfに相当）を、文字数制限に対応させる
	2011/1/16	ver5.5 Calc_Parity 垂直パリティを計算する機能を追加
  ----------------------------------------------------------------------*/

#ifndef HDR_mo_string								// 1度だけインクルードするための工夫。このファイルが一度でも読み込まれると定義されるので、2度は読み込まれない。#endifまで有効。
#define	HDR_mo_string								// このファイルをインクルードしているかはこれが定義されているかでわかる。

#include  <stdlib.h>
#include  <string.h>
#include  <stdarg.h>

#define STR_ERROR_CODE	-1							// 関数でエラーがあったらこれを返す。
//--- ASCII code define ---//
#define	ASC_NUL			0x00						// ヌル文字
#define ASC_CR          0x0d                        // エンターキー : 書き込み点を0文字目へ移す（キャリッジリターン）
#define ASC_LF          0x0a                        // LFCR == \n   : 書き込み点を次の行へ移す
#define ASC_DEL         0x7f                        // デリート
#define ASC_BS          0x08                        // バックスペース
#define ASC_SP          0x20                        // スペース
#define ASC_ESC         0x1b                        // エスケープ
#define ASC_HT			0x09						// 水平タブ

extern int 	Mod_mori0  		(char, char *, int, int, int);
extern void Unti_atoi  		(volatile char *, int, long int);
extern void Unti_atoi_l		(char *, int, long int);
extern void mo_strncpy 		(volatile char *, volatile char *, int);
extern int 	mo_strcpy		(char *, char *);
extern int 	mo_strcount		(char *, char, int);
extern void CharToASCIIcode	(char *,char);
extern char *mo_sprintf_s 	(char *buff, int size, char *format, ...);
extern char Calc_Parity		(char c[], int stop);

/*----------------------------------------------
指定された単文字（例：','）の位置を返す関数
  2006/8/13 森下作成
  
詳細：受信状態によって先頭文字である'$'からの相対距離が変化するパラメータが存在する。そんなパラメータを抜き取る際に必要と感じて作ってみた。

  引数：指定文字，走査対象の文字列，文字列のサイズ，走査を開始する要素数を指定，指定文字何個目か（出現個数のこつ）を指定
  返値：指定個数目の指定文字が存在すればその文字列の何文字目かを要素数で返す。

  用例：ans = Mod_mori0( 'A' , nmea_buff , sizeof(nmea_buff) , 2 , 4);
        nmea_buff[]という文字列から、'A'という単語を探す。３文字目以降から走査を開始して、４つ目の'A'を発見すると、その先頭位置からの数を返す。
        locate1 = Mod_mori0( ',' ,buff,size,0,2);
------------------------------------------------*/
int Mod_mori0(char c,char buff[], int size,int num1,int num2)
{
  int ans,i,temp;

  ans = STR_ERROR_CODE;
  temp = 0;
  for(i = num1 ;i < size; i++){
    if(buff[i] == c)temp++;
    if(temp == num2){
      ans = i;
      break;
    }
  }
  return ans;
}
/*----------------------------------------------------------
受け取った整数をこれまた受け取った配列へ文字として挿入する。左詰めで、格納サイズが足りないとちゃんと文字列化されないので注意。
      
  引数：結果を入れる配列のアドレス、その配列のサイズ、格納したい数字,である。
  注意：配列の容量が足りないとかは対応していませんのであしからず.最後にヌルを入れるのでそのサイズも見てください。

      2006/8/25 改良
	  2007/7/24 アルゴリズムを変更
	  2009/6/12	引数の受け取る数字をlong int型へ変更
-----------------------------------------------------------*/
void Unti_atoi_l(char *data, int len, long int value)
{
  int		wp,rp,i;
  char		c[11];						// unsigned long int に合わせて4200000000が入れば良いのでサイズは11

	i = -1;
	while(value != 0){					// 文字列には、逆順で格納される
		i++;
		c[i] = (char)(value % 10l) + '0';
		value /= 10l;
	}
	if(i < 0){
		c[0] = '0';
		i = 0;
	}
	for(wp = 0, rp = i;wp <= i && wp < len; wp++, rp--)*(data + wp) = c[rp];
	if(wp >= len){						// ヌル文字を格納
		*(data + wp - 1) = 0;
	}else{
		*(data + wp) = 0;
	}
	return;
}
/*----------------------------------------------------------
受け取った整数をこれまた受け取った配列へ文字として挿入する。右詰め。左余白で余った分は'0'詰めとする。
      
  引数：結果を入れる配列のアドレス、その配列のサイズ、格納したい数字,である。
  注意：配列の容量が足りないとかは対応していませんのであしからず.最後にヌルを入れるのでそのサイズも見てください。

      2006/8/25 改良
	  2007/7/24 アルゴリズムを変更
	  2009/6/12	引数の受け取る数字をlong int型へ変更
	  2010/8/27 関数の構造を少し変更
-----------------------------------------------------------*/
void Unti_atoi(volatile char *data, int len, long int value)
{
  long int	temp;
  int		wp;

  for(wp = len - 2; wp >= 0; wp--){
    temp = value % 10l;
	value /= 10l;
    *(data + wp) = (char)(temp + 0x30l);
  }
  *(data + len - 1) = 0;				// 最後をヌルで閉める
  return;
}

/*----------------------------------------------------------------------------------------------
  文字のコピー
  
  作成日時：2007/6/26
-------------------------------------------------------------------------------------------------*/
void mo_strncpy(volatile char *st1,volatile char *st2,int size){
  int i;
  for(i = 0;i < size;i++){
    *(st1 + i) = *(st2 + i);
    //printf("%c\n",*st2);
  }
  return;
}
/*----------------------------------------------------------------------------------------------
  文字のコピー
  
  \0がでるまでコピーする。ヌルまでコピーする。返り値は最期のコピー要素位置(ヌルの位置)
  返り値：コピーした文字数
  作成日時：2007/6/26
-------------------------------------------------------------------------------------------------*/
int mo_strcpy(char *s, char *t){
  int i = -1;

  while((*s++ = *t++))i++;			// コピー後に評価される
  
  return ++i;
}
/*----------------------------------------------------------------------------------------------
  文字の検索…特定文字の計測を行う
  
  作成日時：2009/9/11
  	引数：バッファアドレス、検索文字、バッファサイズ（検索限界を決定）
-------------------------------------------------------------------------------------------------*/
int mo_strcount(char *st, char c, int size){
  int i,k = 0;

  for(i = 0;i < size;i++){
    if(*(st + i) == c)k++;
  }

  return k;
}
/*----------------------------------------------------------------------------------------------
char型の数をHEX形式の文字列へ変換する

  2009/9/12 森下制作
	引数：	c :結果を格納する文字列（のアドレス）,data :HEXにしたい値 
	例：data == 0x34　のとき、c[0] = 0x33,c[1] = 0x34
-------------------------------------------------------------------------------------------------*/
void CharToASCIIcode(char c[], char data){
	char temp;

	temp = (data >> 4) & 0x0f;
	if(temp >= 0x0a){
		temp += 55;
	}else{
		temp += 0x30;
	}
	c[0] = temp;					// 上位のみとする
	temp = data & 0x0f;
	if(temp >= 0x0a){
		temp += 55;					// 小文字なら、87.大文字なら、55
	}else{
		temp += 0x30;
	}
	c[1] = temp;	// 上位ビットをマスク

	return;
}
/*----------------------------------------------------------------------------------------------
ミニsprintf

  2010/02/26	森下制作
  2010/11/11	ヌルで閉める様に変更
  2010/11/15	バッファオーバーラン対策を施して、関数名に_sを付けた
	引数：buff 結果を格納する文字列変数, 文字列変数のサイズ，後はprintfと同じ
	ex. ：(void)mo_sprintf(sent_buff, sizeof(sent_buff), "%s %s %s.\nnum = %d.\n", "first", "sprintf", "test", 5325);	// => "first sprintf test.\nnum = 5325.\n"
-------------------------------------------------------------------------------------------------*/
char *mo_sprintf_s (char *buff, int size, char *format, ...)
{
  short         i = 0;
  char          scratch[16], f, *ptr;
  long 			val = 0, base;
  char          ch, z, n, l, c, m;
  va_list       ap;
  int			k = 0;

  va_start (ap, format);
  for (;;) {
    while ((f = *format++) != '%') {
      if (!f) {
        va_end (ap);
		//return i;						// デフォルトでは時数を返すようになっている
		if(k < size)buff[k++] = 0;		// 2010/11/11 追加
		return buff;					// 格納文字列のポインタを返すように変更
      } else {
		if(k < size)buff[k++] = f;
        if (f == '\n') if(k < size)buff[k++] = '\r';
        i++;
      }
    }
    c = 0;
    // 0で埋める？
    if (!(z = (f = *format++) == '0')) format--;
    // 桁数指定あり？
    f = *format++;
    if (f >= '1' && f <= '9') n = f - '0'; else {n = 0; format--;}
    // 32bit？
    f = *format++;
    l = (f == 'l');
    if (!l) format--;
    m = 0;

    switch (f = *format++) {
      case 'c':
        f = va_arg (ap, int);
        i++;
      default:
        buff[k++] = f;
        continue;
      case 's':
        ptr = va_arg (ap, char *);
		i += strlen(ptr);
		while(*ptr)buff[k++] = *(ptr++);
        continue;
      case 'o':
        val = (long)(l ? va_arg (ap, long) : va_arg (ap, int));
        base = 8L;
        goto CONV_LOOP;
      case 'd':
        if (l) {
          val = va_arg (ap, long);
          if (((long)val) < 0L) {
            m = 1;
            val = - val;
          }
        } else {
          val = va_arg (ap, int);
          if ((short)val < 0) {
            m = 1;
            val = - val;
          }
        }
        base = 10L;
        goto CONV_LOOP;
      case 'b':
        val = (long)(l ? va_arg (ap, long) : va_arg (ap, int));
        base = 2L;
        goto CONV_LOOP;
      case 'u':
        val = (long)(l ? va_arg (ap, long) : va_arg (ap, int));
        base = 10L;
        goto CONV_LOOP;
      case 'X':
        c = 1;
      case 'x':
        base = 16L;
        val = (long)(l ? va_arg (ap, long) : va_arg (ap, int));

      CONV_LOOP:
        for (i = 0; i < 16; i++) scratch[i] = z ? '0' : ' ';
        ptr = scratch + 16;
        *--ptr = 0;
        do {
          ch = (char)(val % base) + '0';
          if (ch > '9') ch += (c ? 'A' : 'a')  - '9' - 1;
          *--ptr = ch;
          val /= base;
        } while (val);
        if (m) *--ptr = '-';
        if (n) ptr = scratch + 15 - n;
		i += strlen(ptr);
		while(*ptr)if(k < size)buff[k++] = *(ptr++);
    }
  }
}

/*----------------------------------------------------------------------------------------------
垂直パリティを計算する

  2011/1/16 森下制作 GPS用のヘッダから、一部改編して移植
	引数：stop 計算する最後の要素番号
-------------------------------------------------------------------------------------------------*/
char Calc_Parity(char c[], int stop){
	char ans = 0;
	int i;

	ans = c[0];
	for(i = 1 ;i <= stop; i++)ans ^= c[i];

	return ans;
}

#endif	// インクルードせねばならん領域のお尻
